""" ./examples/label/severity/select_tests_by_severity.rst """

import pytest
from hamcrest import assert_that, only_contains, any_of, ends_with


@pytest.mark.parametrize(
    ["severities", "expected_tests"],
    [
        (["critical", "minor"],
         [
             "test_vip",
             "test_minor"
         ]),
        (["critical"],
         [
             "test_vip",
         ]),
    ]
)
def test_select_by_severity_level(allured_testdir, severities, expected_tests):
    allured_testdir.parse_docstring_path()

    allured_testdir.run_with_allure("--allure-severities", ",".join(severities))
    test_cases = [test_case["fullName"] for test_case in allured_testdir.allure_report.test_cases]

    assert_that(test_cases, only_contains(
        any_of(
            *[ends_with(name) for name in expected_tests]
        )
    ))



import allure
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status


@allure.issue("352")
def test_pytest_flakes(allured_testdir):
    """
    >>> from os.path import *
    >>> def test_pytest_flakes_example():
    ...     assert True
    """

    allured_testdir.parse_docstring_source()
    allured_testdir.run_with_allure("--flakes")

    assert_that(allured_testdir.allure_report,
                has_test_case("flake-8",
                              with_status("broken")
                              ),

                )

    assert_that(allured_testdir.allure_report,
                has_test_case("test_pytest_flakes_example",
                              with_status("passed")
                              )

                )



# -*- coding: utf-8 -*-

from allure_commons.utils import represent
from hamcrest import assert_that, not_
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_tag


def test_pytest_marker(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.cool
    ... @pytest.mark.stuff
    ... def test_pytest_marker_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_pytest_marker_example",
                              has_tag("cool"),
                              has_tag("stuff")
                              )
                )


def test_show_reserved_pytest_markers_full_decorator(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.usermark1
    ... @pytest.mark.usermark2
    ... @pytest.mark.parametrize("param", ["foo"])
    ... @pytest.mark.skipif(False, reason="reason2")
    ... @pytest.mark.skipif(False, reason="reason1")
    ... def test_show_reserved_pytest_markers_full_decorator_example(param):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case('test_show_reserved_pytest_markers_full_decorator_example[foo]',
                              has_tag("usermark1"),
                              has_tag("usermark2"),
                              has_tag("@pytest.mark.skipif(False, reason='reason1')"),
                              not_(has_tag("@pytest.mark.skipif(False, reason='reason2')")),
                              not_(has_tag("@pytest.mark.parametrize('param', ['foo'])"))
                              )
                )


def test_pytest_xfail_marker(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.xfail(reason='this is unexpect pass')
    ... def test_pytest_xfail_marker_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case('test_pytest_xfail_marker_example',
                              has_tag("@pytest.mark.xfail(reason='this is unexpect pass')"),
                              )
                )


def test_pytest_marker_with_args(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.marker('cool', 'stuff')
    ... def test_pytest_marker_with_args_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_pytest_marker_with_args_example",
                              has_tag("marker('cool', 'stuff')")
                              )
                )


def test_pytest_marker_with_kwargs(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.marker(stuff='cool')
    ... def test_pytest_marker_with_kwargs_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_pytest_marker_with_kwargs_example",
                              has_tag("marker(stuff='cool')")
                              )
                )


def test_pytest_marker_with_kwargs_native_encoding(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.marker(stuff='я')
    ... def test_pytest_marker_with_kwargs_native_encoding_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_pytest_marker_with_kwargs_native_encoding_example",
                              has_tag("marker(stuff=%s)" % represent('я'))
                              )
                )


def test_pytest_marker_with_kwargs_utf_encoding(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.marker(stuff=u'я')
    ... def test_pytest_marker_with_kwargs_utf_encoding_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_pytest_marker_with_kwargs_utf_encoding_example",
                              has_tag("marker(stuff=%s)" % represent('я'))
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_title


def test_dynamic_display_name(executed_docstring_path):
    """ ./examples/display_name/dynamic_display_name.rst """

    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_dynamic_display_name",
                              has_title("It is renamed test")
                              )
                )



""" ./examples/step/step_placeholder.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_step
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains


def test_step_with_args_in_placeholder(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_step_with_args_in_placeholder",
                              has_step("Step with two args: 'first' and 'second'")
                              )
                )


def test_step_with_kwargs_in_placeholder(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_step_with_kwargs_in_placeholder",
                              has_step("Step with two kwargs: '1' and 'second'")
                              )
                )


def test_class_method_as_step(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_class_method_as_step",
                              has_step("Class method step with 'first' and 'second'")
                              )
                )


def test_args_less_than_placeholders(executed_docstring_source):
    """
    >>> import allure

    >>> @allure.step("{0} and {1}")
    ... def step(arg):
    ...     pass

    >>> def test_args_less_than_placeholders_example():
    ...     step(0)
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_args_less_than_placeholders_example",
                              has_status_details(with_message_contains("IndexError: tuple index out of range"))
                              )
                )



# -*- coding: utf-8 -*-
""" ./examples/description/description.rst """

from hamcrest import assert_that, contains_string
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_description, has_description_html


def test_description(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_description",
                              has_description(contains_string("Test description"))
                              )
                )


def test_description_html(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_description_html",
                              has_description_html(contains_string("<h1>Html test description</h1>"))
                              )
                )


def test_docstring_description(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_docstring_description",
                              has_description(contains_string("Docstring"))
                              )
                )


def test_unicode_docstring_description(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_unicode_docstring_description",
                              has_description(contains_string("Докстринг в юникоде"))
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_step
from allure_commons_test.result import with_status
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains
from allure_commons_test.result import with_trace_contains


def test_broken_step(executed_docstring_source):
    """
    >>> import allure

    >>> def test_broken_step_example():
    ...     with allure.step("Step"):
    ...         raise ZeroDivisionError
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_broken_step_example",
                              with_status("broken"),
                              has_status_details(with_message_contains("ZeroDivisionError"),
                                                 with_trace_contains("def test_broken_step_example():")
                                                 ),
                              has_step("Step",
                                       with_status("broken"),
                                       has_status_details(with_message_contains("ZeroDivisionError"),
                                                          with_trace_contains("test_broken_step_example")
                                                          )
                                       )
                              )
                )


def test_pytest_fail_in_step(executed_docstring_source):
    """
    >>> import pytest
    >>> import allure

    >>> def test_pytest_fail_in_step_example():
    ...     with allure.step("Step"):
    ...         pytest.fail()
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_pytest_fail_in_step_example",
                              with_status("failed"),
                              has_status_details(with_message_contains("Failed"),
                                                 with_trace_contains("def test_pytest_fail_in_step_example():")
                                                 ),
                              has_step("Step",
                                       with_status("failed"),
                                       has_status_details(with_message_contains("Failed"),
                                                          with_trace_contains("test_pytest_fail_in_step_example")
                                                          )
                                       )
                              )
                )


def test_pytest_bytes_data_in_assert(executed_docstring_source):
    """
    >>> import allure

    >>> def test_pytest_bytes_data_in_assert_example():
    ...     with allure.step("Step"):
    ...         assert "0\\x82" == 1
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_pytest_bytes_data_in_assert_example",
                              with_status("failed"),
                              has_status_details(with_message_contains("AssertionError: assert \'0\\x82\' == 1"),
                                                 with_trace_contains("def test_pytest_bytes_data_in_assert_example():")
                                                 ),
                              has_step("Step",
                                       with_status("failed"),
                                       has_status_details(
                                           with_message_contains("AssertionError: assert \'0\\x82\' == 1"),
                                           with_trace_contains("test_pytest_bytes_data_in_assert_example")
                                       )
                                       )
                              )
                )



import allure
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status


@allure.feature("Integration")
def test_pytest_docktest(allured_testdir):
    allured_testdir.testdir.makepyfile('''
        def some_func():
            """
            >>> some_func()
            True
            """
            return True

    ''')

    allured_testdir.run_with_allure("--doctest-modules")

    assert_that(allured_testdir.allure_report,
                has_test_case("test_pytest_docktest.some_func",
                              with_status("passed"))
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_label


def test_set_testcase_id_label(executed_docstring_source):
    """
    >>> import allure

    >>> @allure.id(123)
    ... def test_allure_ee_id_label_example():
    ...     pass
    """
    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_allure_ee_id_label_example",
                              has_label("as_id", 123),
                              )
                )



""" ./examples/link/dynamic_link.rst """

import pytest
from hamcrest import assert_that, equal_to
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_link
from allure_commons_test.result import has_issue_link


def test_dynamic_link(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_dynamic_link",
                              has_issue_link("issues/24")
                              )
                )


@pytest.mark.parametrize("link", ["issues/24", "issues/132"])
def test_parametrize_dynamic_link(executed_docstring_path, link):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_parametrize_dynamic_link[{link}]".format(link=link),
                              has_issue_link(link),
                              )
                )


def test_all_links_together(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_all_links_together",
                              has_issue_link("issues/24"),
                              has_issue_link("issues/132"),
                              has_link("allure", name="QAMETA", link_type="docs")
                              )
                )


def test_unique_dynamic_links(executed_docstring_source):
    """
    >>> import allure

    >>> def test_unique_dynamic_links_example():
    ...     allure.dynamic.link("some/unique/dynamic/link")
    ...     allure.dynamic.link("some/unique/dynamic/link")
    """
    assert_that(executed_docstring_source.allure_report.test_cases[0]['links'],
                equal_to([{'url': u'some/unique/dynamic/link', 'type': 'link', 'name': u'some/unique/dynamic/link'}]))



"""./examples/label/severity/class_severity.rst"""

from hamcrest import assert_that, all_of, is_not
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_severity


def test_decorated_class_not_decorated_method(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("TestDecoratedClass#test_not_decorated_method",
                              has_severity("trivial")
                              )
                )


def test_decorated_class_decorated_method(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("TestDecoratedClass#test_decorated_method",
                              all_of(has_severity("minor"),
                                     is_not(has_severity("trivial"))
                                     )
                              )
                )


def test_not_decorated_sub_class_not_decorated_method(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("TestNotDecoratedSubClass#test_not_decorated_method",
                              has_severity("trivial")
                              )
                )


def test_not_decorated_sub_class_decorated_method(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("TestNotDecoratedSubClass#test_decorated_method",
                              all_of(has_severity("critical"),
                                     is_not(has_severity("trivial"))
                                     )
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_attachment


def test_class_method_attachment(executed_docstring_source):
    """
    >>> import allure

    >>> class TestClass(object):
    ...     def test_class_method_attachment(self):
    ...         allure.attach("text", "failed", allure.attachment_type.TEXT)
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_class_method_attachment",
                              has_attachment(name="failed")
                              )
                )



# -*- coding: utf-8 -*-

import pytest
import six
from hamcrest import assert_that
from allure_commons_test.report import has_test_case


@pytest.mark.skipif(six.PY2, reason="python 2.7")
def test_unicode_function_name(executed_docstring_source):
    """
    >>> def test_unicode_func_déjà_vu():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_unicode_func_déjà_vu")
                )


@pytest.mark.skipif(six.PY2, reason="python 2.7")
def test_unicode_method_name(executed_docstring_source):
    """
    >>> class TestCase:
    ...     def test_unicode_method_例子(self):
    ...         pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_unicode_method_例子")
                )


@pytest.mark.skipif(six.PY2, reason="python 2.7")
def test_unicode_class_name(executed_docstring_source):
    """
    >>> class TestCaseПервый:
    ...     def test_method(self):
    ...         pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("TestCaseПервый#test_method")
                )



""" ./examples/attachment/attachment.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_attachment


def test_attach_body_with_default_kwargs(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_attach_body_with_default_kwargs",
                              has_attachment()
                              )
                )


def test_attach_body(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_attach_body",
                              has_attachment(attach_type="application/xml", name="some attachment name")
                              )
                )


def test_attach_file(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_attach_file",
                              has_attachment()
                              )
                )



import allure
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before


@allure.feature("Integration")
def test_lazy_fixture(executed_docstring_source):
    """
    >>> import pytest
    ... from pytest_lazyfixture import lazy_fixture

    >>> @pytest.fixture
    ... def my_lazy_fixture():
    ...     pass

    >>> @pytest.mark.parametrize('param', [lazy_fixture('my_lazy_fixture')])
    ... def test_lazy_fixture_example(param):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_lazy_fixture_example",
                              has_container(executed_docstring_source.allure_report,
                                            has_before("my_lazy_fixture")
                                            ),
                              )
                )


@allure.feature("Integration")
def test_nested_lazy_fixture(executed_docstring_source):
    """
    >>> import pytest
    ... from pytest_lazyfixture import lazy_fixture

    >>> @pytest.fixture
    ... def my_lazy_fixture():
    ...     pass

    >>> @pytest.fixture(params=[lazy_fixture('my_lazy_fixture')])
    ... def my_ordinary_fixture():
    ...     pass

    >>> def test_nested_lazy_fixture_example(my_ordinary_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_nested_lazy_fixture_example",
                              has_container(executed_docstring_source.allure_report,
                                            has_before("my_lazy_fixture")
                                            ),
                              )
                )



import allure
from allure_commons_test.container import has_before
from allure_commons_test.container import has_container
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_step
from hamcrest import assert_that, not_, all_of


@allure.feature("Fixture")
def test_yield_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def yield_fixture():
    ...     pass

    >>> def test_yield_fixture_example(yield_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_yield_fixture_example",
                              has_container(executed_docstring_source.allure_report,
                                            has_before("yield_fixture")
                                            ),
                              )
                )


def test_opened_step_function(executed_docstring_source):
    """
    >>> import allure
    >>> import pytest

    >>> @pytest.fixture()
    ... def yield_fixture():
    ...     with allure.step("Opened step"):
    ...         yield

    >>> def test_opened_step(yield_fixture):
    ...     with allure.step("Body step"):
    ...         pass
    """

    assert_that(
        executed_docstring_source.allure_report,
        has_test_case(
            "test_opened_step",
            all_of(
                has_step("Body step"),
                has_container(
                    executed_docstring_source.allure_report,
                    has_before(
                        "yield_fixture",
                        has_step(
                            "Opened step",
                            not_(has_step("Body step"))
                        )
                    )
                ),
            )
        )
    )



import textwrap
from hamcrest import assert_that
from hamcrest import ends_with
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_package


def test_path_with_dots_test(allured_testdir):
    path = allured_testdir.testdir.mkpydir("path.with.dots")

    path.join("test_path.py").write(
        textwrap.dedent(
            """\
            def test_path_with_dots_test_example():
                pass
            """)
    )

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_path_with_dots_test_example",
                              has_package(ends_with("path.with.dots.test_path"))
                              )
                )


def test_with_no_package(allured_testdir):
    """
    >>> def test_package_less(request):
    ...     pass
    """
    allured_testdir.parse_docstring_source()

    allured_testdir.testdir.makeini("""[pytest]""")
    allured_testdir.run_with_allure(allured_testdir.testdir.tmpdir)

    assert_that(allured_testdir.allure_report,
                has_test_case("test_package_less",
                              has_package("test_with_no_package"))
                )



# -*- coding: utf-8 -*-
""" ./examples/display_name/display_name.rst"""

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_title
from allure_commons_test.label import has_label


def test_display_name(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_display_name",
                              has_title("A some test title")
                              )
                )


def test_display_name_template(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_display_name_template",
                              has_title("A some test title with param False")
                              )
                )


def test_unicode_display_name(executed_docstring_source):
    """
    >>> import allure

    >>> @allure.title(u"Лунтик")
    >>> def test_unicode_display_name_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_unicode_display_name_example", has_title(u"Лунтик"))
                )


def test_unicode_display_name_template(executed_docstring_source):
    """
    >>> import allure
    >>> import pytest

    >>> @allure.title(u"Тест с шаблоном и параметром: {param}")
    ... @pytest.mark.parametrize("param", [False])
    ... def test_unicode_display_name_template_example(param):
    ...     assert param
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_unicode_display_name_template_example",
                              has_title(u"Тест с шаблоном и параметром: False")
                              )
                )


def test_fixture_value_in_display_name(executed_docstring_source):
    """
    >>> import allure
    >>> import pytest

    >>> @pytest.fixture
    ... def fix():
    ...     return 'fixture value'

    >>> @allure.title('title with {fix}')
    ... def test_fixture_value_name(fix):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_fixture_value_name",
                              has_title("title with fixture value")
                              )
                )


def test_display_name_with_features(allured_testdir):
    allured_testdir.testdir.makepyfile("""
        import allure
        import pytest

        @allure.feature('Feature 1')
        @allure.title('Titled test with features')
        @allure.feature('Feature 2')
        def test_feature_label_for_titled_test():
            pass
        """)

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_feature_label_for_titled_test",
                              has_label("feature", "Feature 1"),
                              has_label("feature", "Feature 2"),
                              has_title("Titled test with features")
                              )
                )



import pytest
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_parameter


@pytest.mark.parametrize("param", [True, False])
def test_metafunc_param(executed_docstring_source, param):
    """
    >>> def pytest_generate_tests(metafunc):
    ...     if "metafunc_param" in metafunc.fixturenames:
    ...         metafunc.parametrize("metafunc_param", [True, False])


    >>> def test_metafunc_param_example(metafunc_param):
    ...     assert metafunc_param
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_metafunc_param_example[{param}]".format(param=param),
                              has_parameter("metafunc_param", str(param))
                              )
                )


@pytest.mark.parametrize("param", [True, False])
def test_metafunc_param_with_ids(executed_docstring_source, param):
    """
    >>> def pytest_generate_tests(metafunc):
    ...     if "metafunc_param_with_ids" in metafunc.fixturenames:
    ...         metafunc.parametrize("metafunc_param_with_ids", [True, False], ids=["pass", "fail"])


    >>> def test_metafunc_param_with_ids_example(metafunc_param_with_ids):
    ...     assert metafunc_param_with_ids
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_metafunc_param_with_ids_example[{param}]".format(param="pass" if param else "fail"),
                              has_parameter("metafunc_param_with_ids", str(param))
                              )
                )



import allure
import pytest
from hamcrest import assert_that, ends_with, has_entry
from allure_commons_test.report import has_only_testcases


@allure.issue("292")
@allure.feature("Integration")
@pytest.mark.real_logger
def test_xdist_and_select_test_by_bdd_label(allured_testdir):
    """
    >>> import pytest
    >>> import allure

    >>> @pytest.mark.foo
    ... def test_with_mark_foo():
    ...     print ("hello")

    >>> @allure.feature("boo")
    ... def test_with_feature_boo():
    ...     print ("hello")
    """

    allured_testdir.parse_docstring_source()
    allured_testdir.run_with_allure("-v", "--allure-features=boo", "-n1")

    assert_that(allured_testdir.allure_report,
                has_only_testcases(
                    has_entry("fullName",
                              ends_with("test_with_feature_boo")
                              )
                ))



""" ./examples/attachment/attachment_fixture.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_attachment
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before
from allure_commons_test.container import has_after


def test_fixture_attachment(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_fixture_attachment",
                              has_container(executed_docstring_path.allure_report,
                                            has_before("fixture_with_attachment",
                                                       has_attachment()
                                                       )
                                            )
                              )
                )


def test_fixture_finalizer_attachment(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_fixture_finalizer_attachment",
                              has_container(executed_docstring_path.allure_report,
                                            has_after("fixture_with_attachment_in_finalizer::finalizer",
                                                      has_attachment()
                                                      )
                                            )
                              )
                )



import pytest
import allure
from hamcrest import assert_that, not_
from allure_commons_test.report import has_test_case
from allure_commons_test.container import has_container, has_before, has_after
from allure_commons_test.result import has_step
from itertools import combinations_with_replacement

fixture_scopes = ["session", "module", "class", "function"]


@allure.feature("Fixture")
@pytest.mark.parametrize("first_scope", fixture_scopes)
@pytest.mark.parametrize("second_scope", fixture_scopes)
def test_fixture(allured_testdir, first_scope, second_scope):
    allured_testdir.testdir.makepyfile("""
        import pytest

        @pytest.fixture(scope="{first_scope}")
        def first_fixture():
            pass

        @pytest.fixture(scope="{second_scope}")
        def second_fixture():
            pass

        def test_fixture_example(first_fixture, second_fixture):
            pass
    """.format(first_scope=first_scope, second_scope=second_scope))

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_fixture_example",
                              has_container(allured_testdir.allure_report,
                                            has_before("first_fixture")
                                            ),
                              has_container(allured_testdir.allure_report,
                                            has_before("second_fixture"),
                                            )
                              )
                )


@pytest.mark.parametrize(
    ["parent_scope", "child_scope"],
    list(combinations_with_replacement(fixture_scopes, 2))
)
def test_nested_fixture(allured_testdir, parent_scope, child_scope):
    allured_testdir.testdir.makepyfile("""
        import pytest

        @pytest.fixture(scope="{parent_scope}")
        def parent_fixture():
            pass

        @pytest.fixture(scope="{child_scope}")
        def child_fixture(parent_fixture):
            pass

        def test_nested_fixture_example(child_fixture):
            pass

        def test_fixture_used_in_other_fixtures_example(parent_fixture):
            pass

    """.format(parent_scope=parent_scope, child_scope=child_scope))

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_nested_fixture_example",
                              has_container(allured_testdir.allure_report,
                                            has_before("parent_fixture")
                                            ),
                              has_container(allured_testdir.allure_report,
                                            has_before("child_fixture"),
                                            )
                              )
                )

    assert_that(allured_testdir.allure_report,
                has_test_case("test_fixture_used_in_other_fixtures_example",
                              has_container(allured_testdir.allure_report,
                                            has_before("parent_fixture")
                                            ),
                              not_(has_container(allured_testdir.allure_report,
                                                 has_before("child_fixture"),
                                                 )
                                   )
                              )
                )


@allure.feature("Fixture")
def test_nested_fixtures(executed_docstring_source):
    """
    >>> import pytest

    If we have two fixtures:
    >>> @pytest.fixture
    ... def first_fixture():
    ...     pass


    >>> @pytest.fixture
    ... def second_fixture():
    ...     pass

    And one that uses both previous:
    >>> @pytest.fixture
    ... def child_fixture(first_fixture, second_fixture):
    ...     pass

    For next test, allure report will contain all tree fixtures:
    >>> def test_nested_fixtures_example(child_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_nested_fixtures_example",
                              has_container(executed_docstring_source.allure_report,
                                            has_before("first_fixture")
                                            ),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("second_fixture"),
                                            ),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("child_fixture"),
                                            )
                              )
                )


@allure.feature("Fixture")
def test_fixture_allure_title(allured_testdir):
    allured_testdir.testdir.makepyfile("""
        import pytest
        import allure

        @pytest.fixture
        @allure.title("Allure fixture title")
        def first_fixture():
            pass

        def test_titled_fixture_example(first_fixture):
            pass
    """)

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_titled_fixture_example",
                              has_container(allured_testdir.allure_report,
                                            has_before("Allure fixture title")
                                            )
                              )
                )


@allure.feature("Fixture")
def test_fixture_allure_title_before(allured_testdir):
    allured_testdir.testdir.makepyfile("""
        import pytest
        import allure

        @allure.title("Allure fixture title")
        @pytest.fixture
        def first_fixture():
            pass

        def test_titled_before_fixture_example(first_fixture):
            pass
    """)

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_titled_before_fixture_example",
                              has_container(allured_testdir.allure_report,
                                            has_before("Allure fixture title")
                                            )
                              )
                )


def test_titled_fixture_from_conftest(allured_testdir):
    allured_testdir.testdir.makeconftest("""
        import allure
        import pytest

        @allure.title('Titled fixture before pytest.fixture')
        @pytest.fixture
        def first_fixture():
            pass

        @pytest.fixture
        @allure.title('Titled fixture after pytest.fixture')
        def second_fixture():
            pass
    """)

    allured_testdir.testdir.makepyfile("""
        def test_with_titled_conftest_fixtures(first_fixture, second_fixture):
            pass
    """)

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_with_titled_conftest_fixtures",
                              has_container(allured_testdir.allure_report,
                                            has_before("Titled fixture before pytest.fixture")
                                            ),
                              has_container(allured_testdir.allure_report,
                                            has_before("Titled fixture after pytest.fixture")
                                            )
                              )
                )


def test_fixture_override(allured_testdir):
    allured_testdir.testdir.makeconftest("""
        import pytest
        import allure

        @pytest.fixture
        def my_fixture():
            with allure.step('Step in before in original fixture'):
                pass
            yield
            with allure.step('Step in after in original fixture'):
                pass

    """)

    allured_testdir.testdir.makepyfile("""
        import pytest
        import allure

        @pytest.fixture
        def my_fixture(my_fixture):
            with allure.step('Step in before in redefined fixture'):
                pass
            yield
            with allure.step('Step in after in redefined fixture'):
                pass

        def test_with_redefined_fixture(my_fixture):
            pass
    """)

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_with_redefined_fixture",
                              has_container(allured_testdir.allure_report,
                                            has_before("my_fixture",
                                                       has_step("Step in before in original fixture")
                                                       ),
                                            has_after("my_fixture::0",
                                                      has_step("Step in after in original fixture")
                                                      )
                                            ),
                              has_container(allured_testdir.allure_report,
                                            has_before("my_fixture",
                                                       has_step("Step in before in redefined fixture")
                                                       ),
                                            has_after("my_fixture::0",
                                                      has_step("Step in after in redefined fixture")
                                                      )
                                            ),
                              )
                )



import allure
import pytest
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status


@allure.issue("140")
@allure.feature("Integration")
@pytest.mark.parametrize("countdown", [2, 4])
def test_pytest_rerunfailures(allured_testdir, countdown):
    allured_testdir.testdir.makepyfile("""
        import threading
        import pytest

        back_to_normal = threading.local()

        @pytest.mark.flaky(reruns={countdown})
        def test_pytest_rerunfailures_example():
            countdown = getattr(back_to_normal, "countdown", 3)
            back_to_normal.countdown = countdown - 1
            assert not countdown > 0

    """.format(countdown=countdown))

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_pytest_rerunfailures_example",
                              with_status("failed" if countdown == 2 else "passed"))
                )
